"""
Legal Disclaimer

(c) 2020 Amazon Web Services, Inc. or its affiliates. All Rights Reserved.
This AWS Content is provided subject to the terms of the AWS Customer
Agreement available at https://aws.amazon.com/agreement/ or other written
agreement between Customer and Amazon Web Services, Inc.

Information Block

Checks for Amazon Redshift Cluster to validate service configured inside
or outside VPC network.
API Triggers: CreateCluster, RestoreFromClusterSnapshot
Services: Amazon Cloudwatch Events (trigger), Amazon Simple Notification
Service (Amazon SNS),Amazon Redshift Cluster.
Python 3.6 - AWS Lambda 
"""

import os
import json
import boto3
import logging
from botocore.exceptions import ClientError

OUTBOUND_TOPIC_ARN = os.environ["outbound_topic_arn"]
LOGGING_LEVEL = os.environ["logging_level"]

def lambda_handler(event, context):
    """
    Main Function
    Check Redshift cluster configuration to validate service is configured
    inside AWS VPC network
    """
    setup_logging()
    log.info(json.dumps(event))
    if unexpected_response_handler(event):
        return 'Unexpected Response from AWS'
    if event["detail"]["eventName"] in ["CreateCluster"] or ["RestoreFromClusterSnapshot"]:
        if inside_vpc(event):
            log.info ("No Violation: Redshift cluster configured inside VPC")
        else:
            subject = "Violation - Amazon Redshift Cluster is NON-COMPLIANT"
            message = create_non_compliance_message(
                "Violation: NOT inside VPC Network", event)
            send_violation(OUTBOUND_TOPIC_ARN, message,
                           subject, event, context)


def inside_vpc(event):
    """
    Check Redshift inside VPC Network or Classic Network

    This function check if the newly created Redshift cluster configured inside VPC network
    or Classic network.
    """
    if event["detail"]["responseElements"] is None:
        log.warning('Unexpected Response Element. Triggering violation.')
        return False
    elif "vpcId" in event["detail"]["responseElements"]:
        return True
    elif "vpcSecurityGroupIds" in event["detail"]["requestParameters"]:
        return True
    else:
        return False


def create_non_compliance_message(violation, event):
    """
    Creates non compliance message

    This function will create non-compliance message based on violation report
    """
    message = "Violation - The following Amazon Redshift Cluster was detected to \
    be in violation of the below security compliance \n\n"
    message += "Amazon Redshift Cluster ID : " + \
        event["detail"]["requestParameters"]["clusterIdentifier"] + "\n"
    message += violation + "\n"
    return message


def send_violation(outbound_topic_arn, message, subject, event, context):
    """
    Build A Message

    Creates a message with a first line stating what this violation is checking
    Creates more lines with identifying information about which specific violations were checked
    This is based upon whether or not the specific violation triggered a "Violation" variable
    Then it adds general information regarding the violation
    """
    findsnsregion = outbound_topic_arn.split(":")
    snsregion = findsnsregion[3]
    log.warning('Sending Violation')
    message += '\nAccount: ' + event["account"] + "\n"
    message += "Region: " + event["detail"]["awsRegion"] + "\n"
    message += "\n\n"
    message += "This notification was generated by the Lambda function " + \
        context.invoked_function_arn
    sendclient = boto3.client('sns', region_name=snsregion)
    try:
        response = sendclient.publish(
            TopicArn=outbound_topic_arn,
            Message=message,
            Subject=subject
            )
    except ClientError as err:
        log.info(err)
        return False

def setup_logging():
    """
    Logging Function.

    Creates a global log object and sets its level.
    """
    global log
    log = logging.getLogger()
    log_levels = {'INFO': 20, 'WARNING': 30, 'ERROR': 40}

    if 'logging_level' in os.environ:
        log_level = os.environ['logging_level'].upper()
        if log_level in log_levels:
            log.setLevel(log_levels[log_level])
        else:
            log.setLevel(log_levels['ERROR'])
            log.error("The logging_level environment variable is not set to INFO, WARNING, or \
                        ERROR.  The log level is set to ERROR")
    else:

        log.setLevel(log_levels['ERROR'])
    log.info('Logging setup complete - set to log level ' + str(log.getEffectiveLevel()))

def unexpected_response_handler(event):
    """
    Handles unexpected events.

    This function handles unexpected responses in CloudTrail
    such as limit exceptions or error message response from AWS.
    """
    try:
        if event["detail"]["responseElements"] is None:
            log.info('Null responseElements detected.')
            return True
        elif 'errorCode' in event["detail"]:
            log.warning('errorCode detected, request discarded.')
            return True
        elif 'errorCode' in event["detail"]["responseElements"]:
            log.warning('errorCode detected, request discarded.')
            return True
        else:
            log.info('Passed unexpected response handler.')
            return False
    except KeyError as err:
        log.error(err)